/*
 * Copyright (C) 2005-2011 Alfresco Software Limited.
 *
 * This file is part of Alfresco
 *
 * Alfresco is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Alfresco is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Alfresco. If not, see <http://www.gnu.org/licenses/>.
 */
package org.alfresco.service.cmr.workflow;

/**
 * Client facing API for providing administration information about the
 * {@link WorkflowService}.
 *
 * @author Gavin Cornwell
 * @since 4.0
 */
public interface WorkflowAdminService
{
    /**
     * Determines whether the engine with the given id is enabled.
     * 
     * @param engineId The id of a workflow engine
     * @return true if the engine id is valid and is enabled
     */
    boolean isEngineEnabled(String engineId);
    
    /**
     * Determines whether the JBPM workflow definitions are visible
     * when the JBPM engine is enabled.
     * 
     * NOTE: Workflow definitions can always be retrieved directly 
     * i.e. via name or id
     * 
     * @param engineId The id of a workflow engine
     * @return true if the definitions are visible
     */
    boolean isEngineVisible(String engineId);
}
